package com.jiuqi.util.chinese;

import java.io.Serializable;

public class ChineseDouble implements Cloneable, Serializable {
   private static final char[] INVALID_CHARS = new char[]{' ', '　'};
   private String dbValue;

   public ChineseDouble() {
      this.dbValue = "0";
   }

   public ChineseDouble(double value) {
      this.dbValue = String.valueOf(value);
   }

   public ChineseDouble(String value) {
      if (isDoubleString(value)) {
         this.dbValue = value;
      } else {
         try {
            this.dbValue = valueOf(value);
         } catch (Exception var2) {
            this.dbValue = "0";
         }
      }

   }

   public double doubleValue() {
      return Double.parseDouble(this.dbValue);
   }

   public String chineseValue(int frcLength, boolean showUnit, boolean useBigChar) {
      return parseToChinese(this.dbValue, frcLength, showUnit, useBigChar);
   }

   public static String valueOf(String chineseNumber) throws NotValidChineseNumberException, NoValidCharException {
      boolean isNegtive = false;
      boolean isPercent = false;
      boolean isPermillage = false;
      StringSplitor chars = new StringSplitor(chineseNumber, INVALID_CHARS);
      char thisChar = chars.getNextValidChar();
      if (thisChar == '(' || thisChar == '（') {
         thisChar = chars.getNextValidChar();
         if (thisChar != '负') {
            throw new NotValidChineseNumberException("字符'('后面的字符无效，必须是'负'这个字符！");
         }

         thisChar = chars.getNextValidChar();
         if (thisChar != ')' && thisChar != '）') {
            throw new NotValidChineseNumberException("字符'负'后面的字符无效，必须是')'这个字符！");
         }

         isNegtive = true;
      }

      if (!isNegtive) {
         chars.rollBack();
      }

      thisChar = chars.getNextValidChar();
      if (thisChar != 30334 && thisChar != 20336) {
         if (thisChar == 21315 || thisChar == 20191) {
            thisChar = chars.getNextValidChar();
            if (thisChar != 20998) {
               throw new NotValidChineseNumberException("字符'千'('仟')后面的字符无效，必须是'分'这个字符！");
            }

            thisChar = chars.getNextValidChar();
            if (thisChar != 20043) {
               throw new NotValidChineseNumberException("字符'分'后面的字符无效，必须是'之'这个字符！");
            }

            isPermillage = true;
         }
      } else {
         thisChar = chars.getNextValidChar();
         if (thisChar != 20998) {
            throw new NotValidChineseNumberException("字符'百'('佰')后面的字符无效，必须是'分'这个字符！");
         }

         thisChar = chars.getNextValidChar();
         if (thisChar != 20043) {
            throw new NotValidChineseNumberException("字符'分'后面的字符无效，必须是'之'这个字符！");
         }

         isPercent = true;
      }

      if (!isPercent && !isPermillage) {
         chars.rollBack();
      }

      StringBuffer result = new StringBuffer();
      String left = chars.getLeft();
      int position = left.indexOf(28857);
      if (position != -1) {
         String intPart = left.substring(0, position);
         String frcPart = left.substring(position + 1);
         if (intPart.equals("")) {
            result.append("0");
         } else {
            result.append(ChineseInt.valueOf(intPart));
         }

         result.append(".");
         result.append(ChineseInt.valueOf(frcPart));
      } else {
         result.append(ChineseInt.valueOf(left));
      }

      if (isPercent) {
         result = new StringBuffer(leftMoveDot(result.toString(), 2));
      }

      if (isPermillage) {
         result = new StringBuffer(leftMoveDot(result.toString(), 3));
      }

      if (isNegtive) {
         result.insert(0, "-");
      }

      return result.toString();
   }

   private static String leftMoveDot(String value, int count) {
      StringBuffer number = new StringBuffer(value);
      int position = number.indexOf(".");
      if (position == -1) {
         number.append(".");
         position = number.length() - 1;
      }

      if (count >= position) {
         int loopCount = count + 1 - position;

         for(int i = 0; i < loopCount; ++i) {
            number.insert(0, "0");
            ++position;
         }
      }

      number.deleteCharAt(position);
      number.insert(position - count, ".");
      removeExtraZero(number);
      return number.toString();
   }

   public static String parseToChinese(String value, int frcLength, boolean showUnit, boolean useBigChar) {
      if (isDoubleString(value)) {
         boolean isNegtive = false;
         if (value.startsWith("-")) {
            isNegtive = true;
            value = value.substring(1);
         }

         StringBuffer result = new StringBuffer();
         int dotPos = value.indexOf(46);
         String intPart = null;
         String frcPart = null;
         if (dotPos != -1) {
            intPart = value.substring(0, dotPos);
            frcPart = value.substring(dotPos + 1);
         } else {
            intPart = value;
            frcPart = "";
         }

         if (frcLength >= 0) {
            frcPart = getFrcPart(frcPart, frcLength);
         }

         result.append(ChineseInt.parseToChinese(intPart, showUnit, useBigChar));
         if (!frcPart.equals("")) {
            result.append("点");
            result.append(ChineseInt.parseToChinese(frcPart, false, useBigChar));
         }

         if (isNegtive) {
            result.insert(0, "（负）");
         }

         return result.toString();
      } else {
         throw new IllegalArgumentException("字符串\"" + value + "\"不是一个阿拉伯数字字符串！");
      }
   }

   private static String getFrcPart(String frcPart, int frcLength) {
      int length = frcPart.length();
      if (length >= frcLength) {
         return frcPart.substring(0, frcLength);
      } else {
         StringBuffer sb = new StringBuffer(frcPart);

         for(int i = 0; i < frcLength - length; ++i) {
            sb.append("0");
         }

         return sb.toString();
      }
   }

   public String getPercentForm(int frcLength, boolean showUnit, boolean useBigChar) {
      String value = this.dbValue;
      boolean negative = value.startsWith("-");
      if (negative) {
         value = value.substring(1);
      }

      String result = ChineseDigitUnit.toChineseUnit("100", useBigChar) + "分之" + parseToChinese(rightMoveDot(value, 2), frcLength, showUnit, useBigChar);
      if (negative) {
         result = "（负）" + result;
      }

      return result;
   }

   public String getPermillageForm(int frcLength, boolean showUnit, boolean useBigChar) {
      String value = this.dbValue;
      boolean negative = value.startsWith("-");
      if (negative) {
         value = value.substring(1);
      }

      String result = ChineseDigitUnit.toChineseUnit("1000", useBigChar) + "分之" + parseToChinese(rightMoveDot(value, 3), frcLength, showUnit, useBigChar);
      if (negative) {
         result = "（负）" + result;
      }

      return result;
   }

   private static String rightMoveDot(String value, int count) {
      StringBuffer number = new StringBuffer(value);
      int position = number.indexOf(".");
      if (position == -1) {
         number.append(".");
         position = number.length() - 1;
      }

      if (position + count > number.length() - 1) {
         int loopCount = position + count + 1 - number.length();

         for(int i = 0; i < loopCount; ++i) {
            number.append("0");
         }
      }

      number.deleteCharAt(position);
      number.insert(position + count, ".");
      removeExtraZero(number);
      return number.toString();
   }

   private static void removeExtraZero(StringBuffer number) {
      int dotPos = number.indexOf(".");

      int i;
      char c;
      for(i = 0; i < dotPos - 1; ++i) {
         c = number.charAt(0);
         if (c != '0') {
            break;
         }

         number.deleteCharAt(0);
      }

      dotPos = number.indexOf(".");

      for(i = number.length() - 1; i >= dotPos; --i) {
         c = number.charAt(i);
         if (c != '0' && c != '.') {
            break;
         }

         number.deleteCharAt(number.length() - 1);
      }

   }

   static boolean isDoubleString(String value) {
      int dotPos = value.indexOf(46);
      if (dotPos != -1) {
         String part1 = value.substring(0, dotPos);
         String part2 = value.substring(dotPos + 1);
         return ChineseInt.isIntString(part1) && ChineseInt.isIntString(part2);
      } else {
         return ChineseInt.isIntString(value);
      }
   }

   public Object clone() {
      return new ChineseDouble(this.dbValue);
   }

   public boolean equals(Object o) {
      if (this == o) {
         return true;
      } else if (!(o instanceof ChineseDouble)) {
         return false;
      } else {
         ChineseDouble value = (ChineseDouble)o;
         return this.dbValue.equals(value.dbValue);
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      if (this.dbValue != null) {
         result ^= this.dbValue.hashCode();
      }

      return result;
   }

   public String toString() {
      return this.chineseValue(-1, true, false);
   }
}
